﻿using System;

// Prototype class
abstract class WritingTool
{
    public abstract WritingTool Clone();
    public abstract void Write(string text);
}

// Concrete prototype class: Pen
class Pen : WritingTool
{
    public override WritingTool Clone()
    {
        return (WritingTool)this.MemberwiseClone();
    }

    public override void Write(string text)
    {
        Console.WriteLine("Using a pen to write: " + text);
    }
}

// Concrete prototype class: Pencil
class Pencil : WritingTool
{
    public override WritingTool Clone()
    {
        return (WritingTool)this.MemberwiseClone();
    }

    public override void Write(string text)
    {
        Console.WriteLine("Using a pencil to write: " + text);
    }
}

// Client code
class Program
{
    static void Main(string[] args)
    {
        // Create a prototype object of a pen
        WritingTool penPrototype = new Pen();

        // Clone the prototype to create a new pen
        WritingTool pen = penPrototype.Clone();

        // Use the pen to write some text
        pen.Write("Hello, world!");

        // Create a prototype object of a pencil
        WritingTool pencilPrototype = new Pencil();

        // Clone the prototype to create a new pencil
        WritingTool pencil = pencilPrototype.Clone();

        // Use the pencil to write some text
        pencil.Write("Hello, world!");

        // Clean up memory
        GC.SuppressFinalize(penPrototype);
        GC.SuppressFinalize(pen);
        GC.SuppressFinalize(pencilPrototype);
        GC.SuppressFinalize(pencil);
    }
}
